# Copyright (c) Nuralogix. All rights reserved. Licensed under the MIT license.
# See LICENSE.txt in the project root for license information

class Settings:
    rest_url = "https://api.deepaffex.ai"
    ws_url = "wss://api.deepaffex.ai"
    device_id = ""
    device_token = ""
    device_refresh_token = ""
    role_id = ""
    user_id = ""
    user_token = ""
    user_refresh_token = ""
